% Script evaluating step response for PID controller using both PO and SIMC
% controller tunings
% Written by: Martin S. Foss, fall 2012

clear all
% close all
clc

%Adding "sharedFiles" to MatLab search directory
curDir = pwd;
mainDir = fileparts(curDir);
sharedDir = fullfile(mainDir,'sharedFiles');
addpath(sharedDir);

% for m = 1:8
modelId = 9; %m;
modelName = {'case1','case2','case3','case4','case5','case6',...
             'case7','case8','case9','case10'};
msSet = 1.7;    %define the Ms value

load(fullfile(mainDir,'dataFiles',['resSimc_',...
              num2str(modelName{modelId}),'.mat'])) %loading datafiles

%Collecting PO and SIMC tunings
index = find(res.POpid.minTuning.ms == msSet);
tuning{1} = res.POpid.minTuning.tuning(:,index(1));

index2 = find(res.POpi.minTuning.ms == msSet);
tuning{2} = res.POpi.minTuning.tuning(:,index2(1));

index3 = find(res.simcPID.simcMs <= msSet);
tuning{3} = res.simcPID.simcTuning(:,index3(1));

index4 = find(res.simcPI.simcMs <= msSet);
tuning{4} = res.simcPI.simcTuning(:,index4(1));

gp = model(modelId);        %loading the model

%Extracting parameters from the model
p = pole(gp);               %poles
t1 = abs(p(2)); 	        %time constant 1 (largest)
t2 = abs(p(2)/p(1));        %time constant 2 (smallest) = taud

%% Plotting PO (PID) and SIMC (PID)
ysTime = 1;         %define simaulation parameters
dTime = 20;
stopTime = 40;

figure(modelId)     
clf
colorSet = colormap('lines');

Y = [];     %setting up result matrices
U = [];
T = [];

for i = [1 3]
    subplot(2,1,1)
    gc = controller(tuning{i}(1),tuning{i}(2),tuning{i}(3),t2); 
    sim('simModel')         %running Simulink

    r = plot(t,y);          %plotting output vs. time
    set(r,'Color',colorSet(i,:),'LineWidth',1.5);
    axis([0 stopTime 0 2])
    hold on

    subplot(2,1,2)
    r = plot(t,u);          %plotting input vs. time
    set(r,'Color',colorSet(i,:),'LineWidth',1.5);
    axis([0 40 -.5 4])
    hold on

    Y{i} = y;               %storing results
    U{i} = u;
    T{i} = t;
end

%% Plotting PO (PI) and SIMC (PI)
for k = [2 4]
    subplot(2,1,1)
    [a gc] = controller(tuning{k}(1),tuning{k}(2));
    sim('simModel');
    r = plot(t,y);          %plotting output vs. time
    set(r,'Color',colorSet(k,:),'LineWidth',1.5);
    
    subplot(2,1,2)
    r = plot(t,u);          %plotting inputs vs. time
    set(r,'Color',colorSet(k,:),'LineWidth',1.5);
    
    Y{k} = y;               %storing results
    U{k} = u;
    T{k} = t;
end

subplot(211)
r = plot(t,ys,'--k','linewidth',1.5);       %plotting setpoint
set(r,'Color',[0 0 0],'LineWidth',1.5,'LineStyle','--');

%% Printing info
caseInfo = {'$\frac{\tau_2}{\tau_1}=0.5$ $\frac{\tau_2}{\theta}=0.5$',...
            '$\frac{\tau_2}{\tau_1}=0.8$ $\frac{\tau_2}{\theta}=0.8$',...
            '$\frac{\tau_2}{\tau_1}=0.3$ $\frac{\tau_2}{\theta}=0.3$',...
            '$\frac{\tau_2}{\tau_1}=0.5$ $\frac{\tau_2}{\theta}=1.5$',...
            '$\frac{\tau_2}{\tau_1}=0.8$ $\frac{\tau_2}{\theta}=1.5$',...
            '$\frac{\tau_2}{\tau_1}=0.3$ $\frac{\tau_2}{\theta}=1.5$',...
            '$\frac{\tau_2}{\tau_1}=0.5$ $\frac{\tau_2}{\theta}=2.0$',...
            '$\frac{\tau_2}{\tau_1}=0.8$ $\frac{\tau_2}{\theta}=2.0$',...
            '$\frac{\tau_2}{\tau_1}=0.3$ $\frac{\tau_2}{\theta}=2.0$'};
curveInfo = {'PO (PID)','PO (PI)','SIMC (PID)','SIMC(PI)'};
msString={'$M_s$ $=$ $1.7$'};

stdFont = 16;
infoFontSize = 18;

subplot(211)
ylab = ylabel('Output, $y$');
set(ylab,'interpreter','latex','fontsize',infoFontSize)
set(gca,'fontsize',stdFont,'FontName','Times New Roman')

subplot(212)
ylab = ylabel('Input, $u$');
xlab = xlabel('Time, $t$');
set(xlab,'interpreter','latex','fontsize',infoFontSize)
set(ylab,'interpreter','latex','fontsize',infoFontSize)
set(gca,'fontsize',stdFont,'FontName','Times New Roman')

subplot(211)
[figx figy] = dsxy2figxy(gca,1.9,1.8); % (gca,3,3)
textBoxMs = annotation('textbox',[figx figy .07 .03],'string',...
            msString{1},'interpreter','latex','fontsize',stdFont,...
            'color',[0 0 0],'FitBoxToText','on','LineStyle','none');    

[figx figy] = dsxy2figxy(gca,10,1.75); % (gca,3,3)
textBoxCaseInfo = annotation('textbox',[figx figy .07 .03],'string',...
                  caseInfo{modelId},'interpreter','latex','fontsize',...
                  stdFont,'color',[0 0 0],'FitBoxToText','off',...
                  'LineStyle','none');

[figx figy] = dsxy2figxy(gca,1.9,1.55); % (gca,3,3)
textBoxSetPoint = annotation('textbox',[figx, figy, .07 .03],'string',...
                  '{\bf-}{\bf-} setpoint','interpreter','latex',...
                  'fontsize',stdFont,'color',[0 0 0],'FitBoxToText',...
                  'on','LineStyle','none');

%PO (PID)
y1 = find(Y{1}>=0.8);
[figx figy] = dsxy2figxy(gca,T{1}(y1(1)),Y{1}(y1(1)));
curve1 = annotation('textarrow',[figx+0.05 figx],[figy figy],...
         'string',curveInfo{1},'interpreter','latex','fontsize',stdFont,...
         'color',colorSet(1,:));

%PO (PI)
y2 = find(Y{2}>=0.25);
[figx figy] = dsxy2figxy(gca,T{2}(y2(1)),Y{2}(y2(1)));
curve2 = annotation('textarrow',[figx+0.05 figx],[figy figy],...
         'string',curveInfo{2},'interpreter','latex','fontsize',stdFont,...
         'color',colorSet(2,:));

%SIMC (PID)
y3 = find(Y{3}>=0.5);
[figx figy] = dsxy2figxy(gca,T{3}(y3(1)),Y{3}(y3(1)));
curve3 = annotation('textarrow',[figx+0.05 figx],[figy figy],...
         'string',curveInfo{3},'interpreter','latex','fontsize',stdFont,...
         'color',colorSet(3,:));

%SIMC (PI)
[y4val y4ind] = max(Y{4}(1:150));
[figx figy] = dsxy2figxy(gca,T{4}(y4ind),Y{4}(y4ind));
curve4 = annotation('textarrow',[figx+0.05 figx],[figy+0.02 figy],...
         'string',curveInfo{4},'interpreter','latex','fontsize',stdFont,...
         'color',colorSet(4,:));

subplot(212)
%PO (PID)
if modelId == 7
    [u1val u1ind] = min(U{1});
    [figx figy] = dsxy2figxy(gca,T{1}(u1ind),U{1}(u1ind));
    curve1 = annotation('textarrow',[figx-0.05 figx],[figy figy],...
             'string',curveInfo{1},'interpreter','latex',...
             'fontsize',stdFont,'color',colorSet(1,:));
else
    u1 = find(U{1}(100:end)<=0.2);
    [figx figy] = dsxy2figxy(gca,T{1}(u1(1)+100),U{1}(u1(1)+100));
    curve1 = annotation('textarrow',[figx-0.05 figx],[figy figy],...
             'string',curveInfo{1},'interpreter','latex',...
             'fontsize',stdFont,'color',colorSet(1,:));
end

%PO (PI)
if modelId == 4 || modelId == 5 || modelId == 7 || modelId == 8 || modelId == 9
    [u2val u2ind] = max(U{2});
    [figx figy] = dsxy2figxy(gca,T{2}(u2ind),U{2}(u2ind));
    curve2 = annotation('textarrow',[figx+0.05 figx],[figy+0.03 figy],...
             'string',curveInfo{2},'interpreter','latex',...
             'fontsize',stdFont,'color',colorSet(2,:));
else
    u2 = find(U{2}>=0.70);
    [figx figy] = dsxy2figxy(gca,T{2}(u2(1)),U{2}(u2(1)));
    curve2 = annotation('textarrow',[figx+0.05 figx],[figy-0.02 figy],...
             'string',curveInfo{2},'interpreter','latex','fontsize',...
             stdFont,'color',colorSet(2,:));
end

%SIMC (PID)
u3 = find(U{3}(100:end)<=0.5);
[figx figy] = dsxy2figxy(gca,T{3}(u3(1)+100),U{3}(u3(1)+100));
curve3 = annotation('textarrow',[figx+0.05 figx],[figy+0.02 figy],...
         'string',curveInfo{3},'interpreter','latex',...
         'fontsize',stdFont,'color',colorSet(3,:));

%SIMC (PI)
[u4maxVal u4maxInd] = max(U{4}(1:150));
u4 = find(U{4}(u4maxInd:end)<=1.12);
[figx figy] = dsxy2figxy(gca,T{4}(u4(1)+u4maxInd),U{4}(u4(1)+u4maxInd));
curve4 = annotation('textarrow',[figx+0.05 figx],[figy+0.02 figy],...
         'string',curveInfo{4},'interpreter','latex','fontsize',stdFont,...
         'color',colorSet(4,:));

%% Saving figures
pause
saveas(r,['stepFigures\','stepResponse_',...
          num2str(modelName{modelId}),'.eps'],'psc2')
saveas(r,['stepFigures\','stepResponse_',...
          num2str(modelName{modelId}),'.fig'])
% end

restoredefaultpath