% Script for creating plots comparing SIMC tunings with the PO curve
% Written by: Martin S. Foss, fall 2012

clear all

%Adding "sharedFiles" to MatLab search directory
curDir = pwd;
mainDir = fileparts(curDir);
sharedDir = fullfile(mainDir,'sharedFiles');
addpath(sharedDir);

%%
% for m = 1:8
modelId = 9;
modelName = {'case1','case2','case3','case4','case5','case6',...
             'case7','case8','case9','case10'};
                
    fprintf('Finding SIMC tunings \n')
    fprintf('Case: %g\n',modelId)
    fprintf('***************************\n')
    fprintf('\n')
  
try
    load(fullfile(mainDir,'dataFiles',['resPO_',...
                  num2str(modelName{modelId}),'.mat'])) %loading datafiles
catch me
    ME = MException(me.identifier,...
                    'could not open file, check correct modelId!');
    throw(ME)
end 

minSp = res.POpid.minSp.iae;    %loading iaeWeights;
minD  = res.POpid.minD.iae;
minWeights = [minSp;minD];

manWeights = [.5 .5];   %setting manWeights

tcSpace = 0:0.1:5;      %closed loop time constant search rang for simc

%% SIMC PI-controller
gp = model(modelId);    %get the model

simcJ  = [];            %setting up result matrices
simcMs = [];
simcTuning = [];
i = 1;                  %setting iteration counter

for tc = tcSpace
    [simcTuningTemp, simcGc] = simcPID(gp,tc); %finding the simc PID tuning
    jTemp = costFun(gp,simcGc,minWeights,manWeights);     %finding the cost
    msTemp = ms(gp,simcGc);                          %finding simc ms value
    
    simcJ(i)  = jTemp;          %storing results
    simcMs(i) = msTemp;
    simcTuning(:,i) = simcTuningTemp;
    i = i+1;                    %updating iteration counter
end

%Finding refrence dots for SIMC PID-tuning;
simcRefJ  = [];     %setting up result matrices
simcRefMs = [];

for tc = [.5 1 1.5]
    [simcTuningTemp simcGc] = simcPID(gp,tc);  %finding the simc PID tuning
    jTemp = costFun(gp,simcGc,minWeights,manWeights);     %finding the cost
    msTemp = ms(gp,simcGc);                          %finding simc ms value
 
    simcRefJ  = [simcRefJ jTemp];       %storing results
    simcRefMs = [simcRefMs msTemp];
end

%% SIMC PI-controller
gpPI = modelPI(modelId);    %get the model

simcJPI  = [];              %setting up result matrices
simcMsPI = [];
simcTuningPI = [];
i = 1;                      %setting iteration counter

for tc = tcSpace
    [simcTuningTemp, simcGc] = simcPI(gpPI,tc); %finding the simc PI-tuning
    jTemp = costFun(gp,simcGc,minWeights,manWeights);     %finding the cost
    msTemp = ms(gp,simcGc);                          %finding simc ms value
    
    simcJPI(i)  = jTemp;    %storing results
    simcMsPI(i) = msTemp;
    simcTuningPI(:,i) = simcTuningTemp;
    i = i+1;                %update the iteration counter
end

%Finding refrence dots for SIMC PI-tuning;
simcRefJPI  = [];           %setting up results matrices
simcRefMsPI = [];

for tc = [.5 1 1.5]
    [simcTuningTemp simcGc] = simcPI(gpPI,tc);  %finding the simc PI-tuning
    jTemp = costFun(gp,simcGc,minWeights,manWeights);     %finding the cost
    msTemp = ms(gp,simcGc);                          %finding simc ms value
 
    simcRefJPI  = [simcRefJPI jTemp];       %storing results
    simcRefMsPI = [simcRefMsPI msTemp];
end

%% Plotting the results
colorSet = colormap('lines');
figure(modelId)
clf

h = plot(res.POpid.minTuning.ms,res.POpid.minTuning.costFun,...  %PO(PID)
         res.POpi.minTuning.ms,res.POpi.minTuning.costFun,...    %PO(PI)
         simcMs,simcJ,...                                        %SIMC(PID)
         simcMsPI,simcJPI);                                      %SIMC(PI)
set(h,'LineWidth',1.5)

markerStyles = cellstr(char('d','o','s'));

%Points for SIMC (PID)
hold on
for i = 1:length(simcRefMs)
    h(i) = plot(simcRefMs(i),simcRefJ(i));
    set(h(i),'color',colorSet(3,:),'LineWidth',1.5,'Marker',...
        markerStyles{i},'MarkerSize',10);
end

axis([1 3 0.75 8]); 

%Points for SIMC (PI)
for i = 1:length(simcRefMsPI)
    h(i) = plot(simcRefMsPI(i),simcRefJPI(i));
    set(h(i),'color',colorSet(4,:),'linewidth',1.5,'Marker',...
        markerStyles{i},'MarkerSize',10);
end

%% Printing info
tau2tau1Info = {'$\frac{\tau_2}{\tau_1}=0.5$',...
                '$\frac{\tau_2}{\tau_1}=0.8$',...
                '$\frac{\tau_2}{\tau_1}=0.3$',...
                '$\frac{\tau_2}{\tau_1}=0.5$',...
                '$\frac{\tau_2}{\tau_1}=0.8$',...
                '$\frac{\tau_2}{\tau_1}=0.3$',...
                '$\frac{\tau_2}{\tau_1}=0.5$',...
                '$\frac{\tau_2}{\tau_1}=0.8$',...
                '$\frac{\tau_2}{\tau_1}=0.3$'};
tau2thetaInfo = {'$\frac{\tau_2}{\theta}=0.5$',...
                 '$\frac{\tau_2}{\theta}=0.8$',...
                 '$\frac{\tau_2}{\theta}=0.3$',...
                 '$\frac{\tau_2}{\theta}=1.5$',...
                 '$\frac{\tau_2}{\theta}=1.5$',...
                 '$\frac{\tau_2}{\theta}=1.5$',...
                 '$\frac{\tau_2}{\theta}=2.0$',...
                 '$\frac{\tau_2}{\theta}=2.0$',...
                 '$\frac{\tau_2}{\theta}=2.0$'};
curveInfo = {'PO (PID)','PO (PI)','SIMC (PID)','SIMC(PI)'}; 
pointInfo = {'$\tau_c=0.5\theta$','$\tau_c=\theta$','$\tau_c=1.5\theta$'};

infoFontSize = 18;

xlab = xlabel('Robustness, $M_s$');
ylab = ylabel('Performance, $J(c)$');
set(xlab,'interpreter','latex','fontsize',infoFontSize)
set(ylab,'interpreter','latex','fontsize',infoFontSize)

set(gca,'fontsize',16,'FontName','Times New Roman')

%Model info
[figx figy] = dsxy2figxy(gca,2.4,5.5); 
textBoxTau1Tau2Info = annotation('textbox',[figx figy .07 .03],...
        'string',tau2tau1Info{modelId},'interpreter','latex',...
        'fontsize',infoFontSize,'color',[0 0 0],'FitBoxToText','on',...
        'LineStyle','none');

[figx figy] = dsxy2figxy(gca,2.4,5); % (gca,3,3)
textBoxTau2ThetaInfo = annotation('textbox',[figx figy .07 .03],...
        'string',tau2thetaInfo{modelId},'interpreter','latex',...
        'fontsize',infoFontSize,'color',[0 0 0],'FitBoxToText','on',...
        'LineStyle','none');

%Markers for different tau_c
h = plot(2.4,4.5,'marker',markerStyles{1},'markerSize',10,...
         'linewidth',1.5,'color',[0 0 0]);
[figx figy] = dsxy2figxy(gca,2.4,4.5);
point1 = annotation('textarrow',[figx+0.025 figx+0.015],[figy figy],...
                    'string',pointInfo{1},'interpreter','latex',...
                    'fontsize',infoFontSize,'headstyle','none');

h = plot(2.4,4,'marker',markerStyles{2},'markerSize',10,...
         'linewidth',1.5,'color',[0 0 0]);
[figx figy] = dsxy2figxy(gca,2.4,4);
point2 = annotation('textarrow',[figx+0.025 figx+0.015],[figy figy],...
                    'string',pointInfo{2},'interpreter','latex',...
                    'fontsize',infoFontSize,'headstyle','none');

h = plot(2.4,3.5,'marker',markerStyles{3},'markerSize',10,...
         'linewidth',1.5,'color',[0 0 0]);
[figx figy] = dsxy2figxy(gca,2.4,3.5);
point3 = annotation('textarrow',[figx+0.025 figx+0.015],[figy figy],...
                    'string',pointInfo{3},'interpreter','latex',...
                    'fontsize',infoFontSize,'headstyle','none');

%PO (PID)
if modelId == 8
    x1 = find(res.POpid.minTuning.ms >= 1.4);
else
    x1 = find(res.POpid.minTuning.ms >= 1.5);
end
[figx figy] = dsxy2figxy(gca,res.POpid.minTuning.ms(x1(1)),...
                         res.POpid.minTuning.costFun(x1(1)));
curve1 = annotation('textarrow',[figx-0.03 figx],[figy-0.005 figy],...
                    'string',curveInfo{1},'interpreter','latex',...
                    'fontsize',infoFontSize,'color',colorSet(1,:));

%PO (PI)
x2 = find(res.POpi.minTuning.ms >= 2.4);
[figx figy] = dsxy2figxy(gca,res.POpi.minTuning.ms(x2(1)),...
                         res.POpi.minTuning.costFun(x2(1)));
if modelId < 4
    curve2 = annotation('textarrow',[figx+0.05 figx],[figy+0.05 figy],...
                        'string',curveInfo{2},'interpreter','latex',...
                        'fontsize',infoFontSize,'color',colorSet(2,:));
else
    curve2 = annotation('textarrow',[figx+0.05 figx],[figy-0.05 figy],...
                        'string',curveInfo{2},'interpreter','latex',...
                        'fontsize',infoFontSize,'color',colorSet(2,:));
end

%SIMC (PID)
if modelId == 1 || modelId == 2 || modelId == 3
    x3 = find(simcMs <= 1.20);
elseif modelId == 7
    x3 = find(simcMs <= 1.3);
else
    x3 = find(simcMs <= 1.4);
end
[figx figy] = dsxy2figxy(gca,simcMs(x3(1)),simcJ(x3(1)));
if modelId == 4
    curve3 = annotation('textarrow',[figx+0.05 figx],[figy+0.01 figy],...
                        'string',curveInfo{3},'interpreter','latex',...
                        'fontsize',infoFontSize,'color',colorSet(3,:));
else
    curve3 = annotation('textarrow',[figx+0.05 figx],[figy figy],...
                        'string',curveInfo{3},'interpreter','latex',...
                        'fontsize',infoFontSize,'color',colorSet(3,:));
end

%SIMC (PI)
if modelId == 5 || modelId == 7
    x4 = find(simcMsPI <= 2.1);
else
    x4 = find(simcMsPI <= 2);
end
[figx figy] = dsxy2figxy(gca,simcMsPI(x4(1)),simcJPI(x4(1)));
curve4 = annotation('textarrow',[figx+0.02 figx],[figy+0.03 figy],...
                    'string',curveInfo{4},'interpreter','latex',...
                    'fontsize',infoFontSize,'color',colorSet(4,:));

set(gca,'Layer','top','Box','on')
set(gcf,'paperpositionmode','auto')

%% Storing results and figures 
res.simcPID.simcJ = simcJ;
res.simcPID.simcMs = simcMs;
res.simcPID.simcTuning = simcTuning;

res.simcPI.simcJ = simcJPI;
res.simcPI.simcMs = simcMsPI;
res.simcPI.simcTuning = simcTuningPI;

pause
save([mainDir,'\dataFiles\','resSimc_',...
      num2str(modelName{modelId}),'.mat'],'res')   
save(['simcResults\','resSimc_',num2str(modelName{modelId}),'.mat'],'res')

saveas(h,['SIMCfigures\','simcRes_',...
          num2str(modelName{modelId}),'.eps'],'psc2')
saveas(h,['SIMCfigures\','simcRes_',num2str(modelName{modelId}),'.fig'])
% end

restoredefaultpath